/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.alarmuser.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import vip.xiaonuo.coldchain.modular.alarmuser.dto.ExtendUserDTO;
import vip.xiaonuo.coldchain.modular.alarmuser.entity.AlarmUser;
import vip.xiaonuo.coldchain.modular.alarmuser.param.AlarmUserAddParam;
import vip.xiaonuo.coldchain.modular.alarmuser.param.AlarmUserEditParam;
import vip.xiaonuo.coldchain.modular.alarmuser.param.AlarmUserIdParam;
import vip.xiaonuo.coldchain.modular.alarmuser.param.AlarmUserPageParam;
import vip.xiaonuo.coldchain.modular.alarmuser.service.AlarmUserService;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;

import java.util.List;

/**
 * 报警接收人控制器
 *
 * @author 黄渊昊
 * @date  2024/12/02 15:41
 */
@Tag(name = "报警接收人控制器")
@RestController
@Validated
public class AlarmUserController {

    @Resource
    private AlarmUserService alarmUserService;

    /**
     * 获取报警接收人分页
     *
     * @author 黄渊昊
     * @date  2024/12/02 15:41
     */
    @Operation(summary = "获取报警接收人分页")
    @SaCheckPermission("/coldchain/alarmuser/page")
    @GetMapping("/coldchain/alarmuser/page")
    public CommonResult<Page<ExtendUserDTO>> page(AlarmUserPageParam alarmUserPageParam) {
        return CommonResult.data(alarmUserService.page(alarmUserPageParam));
    }

    /**
     * 添加报警接收人
     *
     * @author 黄渊昊
     * @date  2024/12/02 15:41
     */
    @Operation(summary = "添加报警接收人")
    @CommonLog("添加报警接收人")
    @SaCheckPermission("/coldchain/alarmuser/add")
    @PostMapping("/coldchain/alarmuser/add")
    public CommonResult<String> add(@RequestBody @Valid AlarmUserAddParam alarmUserAddParam) {
        alarmUserService.add(alarmUserAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑报警接收人
     *
     * @author 黄渊昊
     * @date  2024/12/02 15:41
     */
    @Operation(summary = "编辑报警接收人")
    @PostMapping("/coldchain/alarmuser/edit")
    public CommonResult<String> edit(@RequestBody @Valid AlarmUserEditParam alarmUserEditParam) {
        alarmUserService.edit(alarmUserEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除报警接收人
     *
     * @author 黄渊昊
     * @date  2024/12/02 15:41
     */
    @Operation(summary = "删除报警接收人")
    @CommonLog("删除报警接收人")
    @SaCheckPermission("/coldchain/alarmuser/delete")
    @PostMapping("/coldchain/alarmuser/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<AlarmUserIdParam> alarmUserIdParamList) {
        alarmUserService.delete(alarmUserIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取报警接收人详情
     *
     * @author 黄渊昊
     * @date  2024/12/02 15:41
     */
    @Operation(summary = "获取报警接收人详情")
    @SaCheckPermission("/coldchain/alarmuser/detail")
    @GetMapping("/coldchain/alarmuser/detail")
    public CommonResult<AlarmUser> detail(@Valid AlarmUserIdParam alarmUserIdParam) {
        return CommonResult.data(alarmUserService.detail(alarmUserIdParam));
    }

    /**
     * 根据登录人所属组织获取
     */
    @Operation(summary = "根据登录人所属组织获取")
    @SaCheckPermission("/coldchain/alarmuser/list")
    @GetMapping("/coldchain/alarmuser/list")
    public CommonResult<List<ExtendUserDTO>> list() {
        return CommonResult.data(alarmUserService.myList());
    }

    /**
     * 获取微信用户信息
     */
    @Operation(summary = "获取微信用户信息")
    @GetMapping("/coldchain/alarmuser/getUserInfo")
    public CommonResult<String> getUserInfo(@RequestParam String code, @RequestParam(required = false) String orgId, @RequestParam(required = false) String name) {
        return CommonResult.data(alarmUserService.getUserInfo(code, orgId, name));
    }
}
