/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.alarmuser.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import vip.xiaonuo.coldchain.modular.alarmuser.dto.ExtendUserDTO;
import vip.xiaonuo.coldchain.modular.alarmuser.entity.AlarmUser;
import vip.xiaonuo.coldchain.modular.alarmuser.param.AlarmUserAddParam;
import vip.xiaonuo.coldchain.modular.alarmuser.param.AlarmUserEditParam;
import vip.xiaonuo.coldchain.modular.alarmuser.param.AlarmUserIdParam;
import vip.xiaonuo.coldchain.modular.alarmuser.param.AlarmUserPageParam;

import java.util.List;

/**
 * 报警接收人Service接口
 *
 * @author 黄渊昊
 * @date  2024/12/02 15:41
 **/
public interface AlarmUserService extends IService<AlarmUser> {

    /**
     * 获取报警接收人分页
     *
     * @author 黄渊昊
     * @date 2024/12/02 15:41
     */
    Page<ExtendUserDTO> page(AlarmUserPageParam alarmUserPageParam);

    /**
     * 添加报警接收人
     *
     * @author 黄渊昊
     * @date  2024/12/02 15:41
     */
    void add(AlarmUserAddParam alarmUserAddParam);

    /**
     * 编辑报警接收人
     *
     * @author 黄渊昊
     * @date  2024/12/02 15:41
     */
    void edit(AlarmUserEditParam alarmUserEditParam);

    /**
     * 删除报警接收人
     *
     * @author 黄渊昊
     * @date  2024/12/02 15:41
     */
    void delete(List<AlarmUserIdParam> alarmUserIdParamList);

    /**
     * 获取报警接收人详情
     *
     * @author 黄渊昊
     * @date  2024/12/02 15:41
     */
    AlarmUser detail(AlarmUserIdParam alarmUserIdParam);

    /**
     * 获取报警接收人详情
     *
     * @author 黄渊昊
     * @date  2024/12/02 15:41
     **/
    AlarmUser queryEntity(String id);

    /**
     * 根据登录人所属组织获取
     */
    List<ExtendUserDTO> myList();

    /**
     * 获取微信用户信息
     * @param code
     */
    String getUserInfo(String code, String orgId, String name);

    /**
     * 关注微信号，如果openid不存在则创建用户
     */
    AlarmUser subscribe(String openid);

    /**
     * 取消关注微信号
     */
    void unsubscribe(String openid);

    /**
     * 根据openId获取用户信息
     */
    List<AlarmUser> getByOpenId(String openId);
}
