/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.monitordevice.param;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Getter;
import lombok.Setter;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

/**
 * 采集器管理添加参数
 *
 * @author 黄渊昊
 * @date 2024/11/13 16:55
 **/
@Getter
@Setter
public class MonitorDeviceAddParam {
    /**
     * 设备名称
     */
    @Schema(description = "设备名称")
    private String deviceName;

    /**
     * 设备编码
     */
    @Schema(description = "设备编码")
    @NotNull(message = "设备编码不能为空")
    private String deviceCode;

    /**
     * 设备型号
     */
    @Schema(description = "设备型号")
    @NotNull(message = "设备型号不能为空")
    private String modelName;

    /**
     * 设备状态（1:正常，2：闲置）
     */
    @Schema(description = "设备状态（1:正常，2：闲置）")
    private String status = "1";

    /**
     * 传感器路数
     */
    @Schema(description = "传感器路数")
    @NotNull(message = "传感器路数不能为空")
    @Size(min = 1, max = 4, message = "传感器路数必须在 1 到 4 之间")
    private Integer sensorCount;

    /**
     * 排序码
     */
    @Schema(description = "排序码")
    private Integer sortCode;

    /**
     * mac
     */
    @Schema(description = "mac")
    private String mac;

    /**
     * ip
     */
    @Schema(description = "ip")
    private String ip;

    /**
     * 报警上限
     */
    @Schema(description = "报警上限")
    private Float temperatureUp;

    /**
     * 报警上限
     */
    @Schema(description = "报警上限")
    private Float humidityUp;

    /**
     * 报警上限
     */
    @Schema(description = "报警上限")
    private Float co2Up;

    /**
     * 报警下限
     */
    @Schema(description = "报警下限")
    private Float temperatureDown;

    /**
     * 报警下限
     */
    @Schema(description = "报警下限")
    private Float humidityDown;

    /**
     * 报警下限
     */
    @Schema(description = "报警下限")
    private Float co2Down;

    /** 供电类型 */
    @Schema(description = "供电类型")
    private String powerType;
}
