/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.monitordevice.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.github.jfcloud.influxdb.flux.AggregationWindow;
import com.github.jfcloud.influxdb.flux.QueryCondition;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.web.multipart.MultipartFile;
import rk.netDevice.sdk.p2.ParamData;
import rk.netDevice.sdk.p2.ParamIdsData;
import vip.xiaonuo.coldchain.core.bean.influxdb.SensorData;
import vip.xiaonuo.coldchain.modular.monitordevice.entity.MonitorDevice;
import vip.xiaonuo.coldchain.modular.monitordevice.param.MonitorDeviceAddParam;
import vip.xiaonuo.coldchain.modular.monitordevice.param.MonitorDeviceEditParam;
import vip.xiaonuo.coldchain.modular.monitordevice.param.MonitorDeviceIdParam;
import vip.xiaonuo.coldchain.modular.monitordevice.param.MonitorDevicePageParam;
import vip.xiaonuo.coldchain.modular.monitordevicetype.entity.CountEntity;

import java.time.Instant;
import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * 采集器管理Service接口
 *
 * @author 黄渊昊
 * @date 2024/11/13 16:55
 **/
public interface MonitorDeviceService extends IService<MonitorDevice> {

    /**
     * 获取采集器管理分页
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:55
     */
    Page<?> page(MonitorDevicePageParam monitorDevicePageParam);

    /**
     * 添加采集器管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:55
     */
    void add(MonitorDeviceAddParam monitorDeviceAddParam);

    /**
     * 编辑采集器管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:55
     */
    void edit(MonitorDeviceEditParam monitorDeviceEditParam);

    /**
     * 删除采集器管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:55
     */
    void delete(List<MonitorDeviceIdParam> monitorDeviceIdParamList);

    /**
     * 获取采集器管理详情
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:55
     */
    MonitorDevice detail(MonitorDeviceIdParam monitorDeviceIdParam);

    /**
     * 获取采集器管理详情
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:55
     **/
    MonitorDevice queryEntity(String id);

    boolean updateParamIds(ParamIdsData data);


    boolean updateParameters(ParamData data);

    boolean updateLastHeartbeatTimeByDeviceId(Integer deviceId, Date timestamp);

    boolean updateLastLoginTimeByDeviceId(Integer deviceId, Date timestamp);

    List<SensorData> queryDataByDeviceIdAndRoads(String deviceId, Integer sensorRoute, String startTime, String endTime, String field, AggregationWindow aggregationWindow);


    SensorData queryLatestDataByDeviceIdAndRoads(String deviceId, Integer sensorRoute);

    List<SensorData> querySensorData(String deviceId, int roads, Instant startTime, Instant endTime, List<QueryCondition> queryConditions);

    List<MonitorDevice> myList(Boolean isAll,String sensorType);

    boolean getByDeviceName(String deviceName);

    boolean getByDeviceCode(String deviceCode);

    /**
     * 获取首页统计信息
     */
    CountEntity getIndexCount();

    MonitorDevice queryByDeviceCode(Integer deviceId);

    /**
     * 下载导入模版
     * @param response
     */
    void downloadTemplate(HttpServletResponse response);

    /**
     * 导入数据
     * @param file
     * @return
     */
    void importMonitorDevice(MultipartFile file);

    Set<String> getAllDeviceCodes();
}
