/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.monitordeviceparameters.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.coldchain.modular.monitordeviceparameters.entity.MonitorDeviceParameters;
import vip.xiaonuo.coldchain.modular.monitordeviceparameters.param.MonitorDeviceParametersAddParam;
import vip.xiaonuo.coldchain.modular.monitordeviceparameters.param.MonitorDeviceParametersEditParam;
import vip.xiaonuo.coldchain.modular.monitordeviceparameters.param.MonitorDeviceParametersIdParam;
import vip.xiaonuo.coldchain.modular.monitordeviceparameters.param.MonitorDeviceParametersPageParam;
import vip.xiaonuo.coldchain.modular.monitordeviceparameters.service.MonitorDeviceParametersService;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;

import java.util.List;

/**
 * 设备参数及其配置控制器
 *
 * @author 黄渊昊
 * @date 2024/11/13 17:03
 */
@Tag(name = "设备参数及其配置控制器")
@RestController
@Validated
public class MonitorDeviceParametersController {

    @Resource
    private MonitorDeviceParametersService monitorDeviceParametersService;

    /**
     * 获取设备参数及其配置分页
     *
     * @author 黄渊昊
     * @date 2024/11/13 17:03
     */
    @Operation(summary = "获取设备参数及其配置分页")
    @SaCheckPermission("/coldchain/monitordeviceparameters/page")
    @GetMapping("/coldchain/monitordeviceparameters/page")
    public CommonResult<Page<MonitorDeviceParameters>> page(MonitorDeviceParametersPageParam monitorDeviceParametersPageParam) {
        return CommonResult.data(monitorDeviceParametersService.page(monitorDeviceParametersPageParam));
    }

    /**
     * 添加设备参数及其配置
     *
     * @author 黄渊昊
     * @date 2024/11/13 17:03
     */
    @Operation(summary = "添加设备参数及其配置")
    @CommonLog("添加设备参数及其配置")
    @SaCheckPermission("/coldchain/monitordeviceparameters/add")
    @PostMapping("/coldchain/monitordeviceparameters/add")
    public CommonResult<String> add(@RequestBody @Valid MonitorDeviceParametersAddParam monitorDeviceParametersAddParam) {
        monitorDeviceParametersService.add(monitorDeviceParametersAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑设备参数及其配置
     *
     * @author 黄渊昊
     * @date 2024/11/13 17:03
     */
    @Operation(summary = "编辑设备参数及其配置")
    @CommonLog("编辑设备参数及其配置")
    @SaCheckPermission("/coldchain/monitordeviceparameters/edit")
    @PostMapping("/coldchain/monitordeviceparameters/edit")
    public CommonResult<String> edit(@RequestBody @Valid MonitorDeviceParametersEditParam monitorDeviceParametersEditParam) {
        monitorDeviceParametersService.edit(monitorDeviceParametersEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除设备参数及其配置
     *
     * @author 黄渊昊
     * @date 2024/11/13 17:03
     */
    @Operation(summary = "删除设备参数及其配置")
    @CommonLog("删除设备参数及其配置")
    @SaCheckPermission("/coldchain/monitordeviceparameters/delete")
    @PostMapping("/coldchain/monitordeviceparameters/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                       List<MonitorDeviceParametersIdParam> monitorDeviceParametersIdParamList) {
        monitorDeviceParametersService.delete(monitorDeviceParametersIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取设备参数及其配置详情
     *
     * @author 黄渊昊
     * @date 2024/11/13 17:03
     */
    @Operation(summary = "获取设备参数及其配置详情")
    @SaCheckPermission("/coldchain/monitordeviceparameters/detail")
    @GetMapping("/coldchain/monitordeviceparameters/detail")
    public CommonResult<MonitorDeviceParameters> detail(@Valid MonitorDeviceParametersIdParam monitorDeviceParametersIdParam) {
        return CommonResult.data(monitorDeviceParametersService.detail(monitorDeviceParametersIdParam));
    }
}
