/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.monitordeviceparameters.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.coldchain.modular.monitordeviceparameters.entity.MonitorDeviceParameters;
import vip.xiaonuo.coldchain.modular.monitordeviceparameters.mapper.MonitorDeviceParametersMapper;
import vip.xiaonuo.coldchain.modular.monitordeviceparameters.param.MonitorDeviceParametersAddParam;
import vip.xiaonuo.coldchain.modular.monitordeviceparameters.param.MonitorDeviceParametersEditParam;
import vip.xiaonuo.coldchain.modular.monitordeviceparameters.param.MonitorDeviceParametersIdParam;
import vip.xiaonuo.coldchain.modular.monitordeviceparameters.param.MonitorDeviceParametersPageParam;
import vip.xiaonuo.coldchain.modular.monitordeviceparameters.service.MonitorDeviceParametersService;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;

import java.util.List;

/**
 * 设备参数及其配置Service接口实现类
 *
 * @author 黄渊昊
 * @date 2024/11/13 17:03
 **/
@Service
public class MonitorDeviceParametersServiceImpl extends ServiceImpl<MonitorDeviceParametersMapper, MonitorDeviceParameters> implements MonitorDeviceParametersService {

    @Override
    public Page<MonitorDeviceParameters> page(MonitorDeviceParametersPageParam monitorDeviceParametersPageParam) {
        QueryWrapper<MonitorDeviceParameters> queryWrapper = new QueryWrapper<MonitorDeviceParameters>().checkSqlInjection();
        if (ObjectUtil.isNotEmpty(monitorDeviceParametersPageParam.getParamType())) {
            queryWrapper.lambda().eq(MonitorDeviceParameters::getParamType, monitorDeviceParametersPageParam.getParamType());
        }
        if (ObjectUtil.isNotEmpty(monitorDeviceParametersPageParam.getValue())) {
            queryWrapper.lambda().like(MonitorDeviceParameters::getValue, monitorDeviceParametersPageParam.getValue());
        }
        if (ObjectUtil.isAllNotEmpty(monitorDeviceParametersPageParam.getSortField(), monitorDeviceParametersPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(monitorDeviceParametersPageParam.getSortOrder());
            queryWrapper.orderBy(true, monitorDeviceParametersPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(monitorDeviceParametersPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(MonitorDeviceParameters::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(MonitorDeviceParametersAddParam monitorDeviceParametersAddParam) {
        MonitorDeviceParameters monitorDeviceParameters = BeanUtil.toBean(monitorDeviceParametersAddParam, MonitorDeviceParameters.class);
        this.save(monitorDeviceParameters);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(MonitorDeviceParametersEditParam monitorDeviceParametersEditParam) {
        MonitorDeviceParameters monitorDeviceParameters = this.queryEntity(monitorDeviceParametersEditParam.getId());
        BeanUtil.copyProperties(monitorDeviceParametersEditParam, monitorDeviceParameters);
        this.updateById(monitorDeviceParameters);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<MonitorDeviceParametersIdParam> monitorDeviceParametersIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(monitorDeviceParametersIdParamList, MonitorDeviceParametersIdParam::getId));
    }

    @Override
    public MonitorDeviceParameters detail(MonitorDeviceParametersIdParam monitorDeviceParametersIdParam) {
        return this.queryEntity(monitorDeviceParametersIdParam.getId());
    }

    @Override
    public MonitorDeviceParameters queryEntity(String id) {
        MonitorDeviceParameters monitorDeviceParameters = this.getById(id);
        if (ObjectUtil.isEmpty(monitorDeviceParameters)) {
            throw new CommonException("设备参数及其配置不存在，id值为：{}", id);
        }
        return monitorDeviceParameters;
    }
}
