/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.monitordevicetype.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.coldchain.modular.monitordevicetype.entity.MonitorDeviceType;
import vip.xiaonuo.coldchain.modular.monitordevicetype.param.MonitorDeviceTypeAddParam;
import vip.xiaonuo.coldchain.modular.monitordevicetype.param.MonitorDeviceTypeEditParam;
import vip.xiaonuo.coldchain.modular.monitordevicetype.param.MonitorDeviceTypeIdParam;
import vip.xiaonuo.coldchain.modular.monitordevicetype.param.MonitorDeviceTypePageParam;
import vip.xiaonuo.coldchain.modular.monitordevicetype.service.MonitorDeviceTypeService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 监控设备型号控制器
 *
 * @author 黄渊昊
 * @date  2024/11/25 10:15
 */
@Tag(name = "监控设备型号控制器")
@RestController
@Validated
public class MonitorDeviceTypeController {

    @Resource
    private MonitorDeviceTypeService monitorDeviceTypeService;

    /**
     * 获取监控设备型号分页
     *
     * @author 黄渊昊
     * @date  2024/11/25 10:15
     */
    @Operation(summary = "获取监控设备型号分页")
    @SaCheckPermission("/coldchain/monitordevicetype/page")
    @GetMapping("/coldchain/monitordevicetype/page")
    public CommonResult<Page<MonitorDeviceType>> page(MonitorDeviceTypePageParam monitorDeviceTypePageParam) {
        return CommonResult.data(monitorDeviceTypeService.page(monitorDeviceTypePageParam));
    }

    /**
     * 添加监控设备型号
     *
     * @author 黄渊昊
     * @date  2024/11/25 10:15
     */
    @Operation(summary = "添加监控设备型号")
    @CommonLog("添加监控设备型号")
    @SaCheckPermission("/coldchain/monitordevicetype/add")
    @PostMapping("/coldchain/monitordevicetype/add")
    public CommonResult<String> add(@RequestBody @Valid MonitorDeviceTypeAddParam monitorDeviceTypeAddParam) {
        monitorDeviceTypeService.add(monitorDeviceTypeAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑监控设备型号
     *
     * @author 黄渊昊
     * @date  2024/11/25 10:15
     */
    @Operation(summary = "编辑监控设备型号")
    @CommonLog("编辑监控设备型号")
    @SaCheckPermission("/coldchain/monitordevicetype/edit")
    @PostMapping("/coldchain/monitordevicetype/edit")
    public CommonResult<String> edit(@RequestBody @Valid MonitorDeviceTypeEditParam monitorDeviceTypeEditParam) {
        monitorDeviceTypeService.edit(monitorDeviceTypeEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除监控设备型号
     *
     * @author 黄渊昊
     * @date  2024/11/25 10:15
     */
    @Operation(summary = "删除监控设备型号")
    @CommonLog("删除监控设备型号")
    @SaCheckPermission("/coldchain/monitordevicetype/delete")
    @PostMapping("/coldchain/monitordevicetype/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<MonitorDeviceTypeIdParam> monitorDeviceTypeIdParamList) {
        monitorDeviceTypeService.delete(monitorDeviceTypeIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取监控设备型号详情
     *
     * @author 黄渊昊
     * @date  2024/11/25 10:15
     */
    @Operation(summary = "获取监控设备型号详情")
    @SaCheckPermission("/coldchain/monitordevicetype/detail")
    @GetMapping("/coldchain/monitordevicetype/detail")
    public CommonResult<MonitorDeviceType> detail(@Valid MonitorDeviceTypeIdParam monitorDeviceTypeIdParam) {
        return CommonResult.data(monitorDeviceTypeService.detail(monitorDeviceTypeIdParam));
    }

    /**
     * 获取监控设备类型列表
     */
    @Operation(summary = "获取监控设备类型列表")
    @SaCheckPermission("/coldchain/monitordevicetype/list")
    @GetMapping("/coldchain/monitordevicetype/list")
    public CommonResult<List<MonitorDeviceType>> list(String sensorType) {
        return CommonResult.data(monitorDeviceTypeService.myList(sensorType));
    }

    /**
     * 检查类型名是否已注册
     */
    @Operation(summary = "检查类型名是否已注册")
    @SaCheckPermission("/coldchain/monitordevicetype/getByTypeName")
    @GetMapping("/coldchain/monitordevicetype/getByTypeName")
    public CommonResult<String> getByTypeName(String name) {
        boolean b = monitorDeviceTypeService.getByName(name);
        if (b) {
            return CommonResult.error("类型名" + name + "已注册使用：{}");
        }
        return CommonResult.ok();
    }

    /**
     * 检查类型编码是否已注册
     */
    @Operation(summary = "检查类型编码是否已注册")
    @SaCheckPermission("/coldchain/monitordevicetype/getByTypeCode")
    @GetMapping("/coldchain/monitordevicetype/getByTypeCode")
    public CommonResult<String> getByTypeCode(String code) {
        boolean b = monitorDeviceTypeService.getByCode(code);
        if (b) {
            return CommonResult.error("类型编码" + code + "已注册使用");
        }
        return CommonResult.ok();
    }
}
