/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.monitormonitortarget.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.coldchain.modular.monitormonitortarget.entity.MonitorMonitorTarget;
import vip.xiaonuo.coldchain.modular.monitormonitortarget.param.MonitorMonitorTargetAddParam;
import vip.xiaonuo.coldchain.modular.monitormonitortarget.param.MonitorMonitorTargetEditParam;
import vip.xiaonuo.coldchain.modular.monitormonitortarget.param.MonitorMonitorTargetIdParam;
import vip.xiaonuo.coldchain.modular.monitormonitortarget.param.MonitorMonitorTargetPageParam;
import vip.xiaonuo.coldchain.modular.monitormonitortarget.service.MonitorMonitorTargetService;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;

import java.util.List;

/**
 * 采集器与监控对象关系控制器
 *
 * @author 黄渊昊
 * @date 2024/11/15 10:45
 */
@Tag(name = "采集器与监控对象关系控制器")
@RestController
@Validated
public class MonitorMonitorTargetController {

    @Resource
    private MonitorMonitorTargetService monitorMonitorTargetService;

    /**
     * 获取采集器与监控对象关系分页
     *
     * @author 黄渊昊
     * @date 2024/11/15 10:45
     */
    @Operation(summary = "获取采集器与监控对象关系分页")
    @SaCheckPermission("/coldchain/monitormonitortarget/page")
    @GetMapping("/coldchain/monitormonitortarget/page")
    public CommonResult<Page<MonitorMonitorTarget>> page(MonitorMonitorTargetPageParam monitorMonitorTargetPageParam) {
        return CommonResult.data(monitorMonitorTargetService.page(monitorMonitorTargetPageParam));
    }

    /**
     * 添加采集器与监控对象关系
     *
     * @author 黄渊昊
     * @date 2024/11/15 10:45
     */
    @Operation(summary = "添加采集器与监控对象关系")
    @CommonLog("添加采集器与监控对象关系")
    @SaCheckPermission("/coldchain/monitormonitortarget/add")
    @PostMapping("/coldchain/monitormonitortarget/add")
    public CommonResult<String> add(@RequestBody @Valid MonitorMonitorTargetAddParam monitorMonitorTargetAddParam) {
        monitorMonitorTargetService.add(monitorMonitorTargetAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑采集器与监控对象关系
     *
     * @author 黄渊昊
     * @date 2024/11/15 10:45
     */
    @Operation(summary = "编辑采集器与监控对象关系")
    @CommonLog("编辑采集器与监控对象关系")
    @SaCheckPermission("/coldchain/monitormonitortarget/edit")
    @PostMapping("/coldchain/monitormonitortarget/edit")
    public CommonResult<String> edit(@RequestBody @Valid MonitorMonitorTargetEditParam monitorMonitorTargetEditParam) {
        monitorMonitorTargetService.edit(monitorMonitorTargetEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除采集器与监控对象关系
     *
     * @author 黄渊昊
     * @date 2024/11/15 10:45
     */
    @Operation(summary = "删除采集器与监控对象关系")
    @CommonLog("删除采集器与监控对象关系")
    @SaCheckPermission("/coldchain/monitormonitortarget/delete")
    @PostMapping("/coldchain/monitormonitortarget/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                       List<MonitorMonitorTargetIdParam> monitorMonitorTargetIdParamList) {
        monitorMonitorTargetService.delete(monitorMonitorTargetIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取采集器与监控对象关系详情
     *
     * @author 黄渊昊
     * @date 2024/11/15 10:45
     */
    @Operation(summary = "获取采集器与监控对象关系详情")
    @SaCheckPermission("/coldchain/monitormonitortarget/detail")
    @GetMapping("/coldchain/monitormonitortarget/detail")
    public CommonResult<MonitorMonitorTarget> detail(@Valid MonitorMonitorTargetIdParam monitorMonitorTargetIdParam) {
        return CommonResult.data(monitorMonitorTargetService.detail(monitorMonitorTargetIdParam));
    }
}
