/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.monitormonitortarget.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.coldchain.modular.monitormonitortarget.entity.MonitorMonitorTarget;
import vip.xiaonuo.coldchain.modular.monitormonitortarget.mapper.MonitorMonitorTargetMapper;
import vip.xiaonuo.coldchain.modular.monitormonitortarget.param.MonitorMonitorTargetAddParam;
import vip.xiaonuo.coldchain.modular.monitormonitortarget.param.MonitorMonitorTargetEditParam;
import vip.xiaonuo.coldchain.modular.monitormonitortarget.param.MonitorMonitorTargetIdParam;
import vip.xiaonuo.coldchain.modular.monitormonitortarget.param.MonitorMonitorTargetPageParam;
import vip.xiaonuo.coldchain.modular.monitormonitortarget.service.MonitorMonitorTargetService;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;

import java.util.List;

/**
 * 采集器与监控对象关系Service接口实现类
 *
 * @author 黄渊昊
 * @date 2024/11/15 10:45
 **/
@Service
public class MonitorMonitorTargetServiceImpl extends ServiceImpl<MonitorMonitorTargetMapper, MonitorMonitorTarget> implements MonitorMonitorTargetService {

    @Override
    public Page<MonitorMonitorTarget> page(MonitorMonitorTargetPageParam monitorMonitorTargetPageParam) {
        QueryWrapper<MonitorMonitorTarget> queryWrapper = new QueryWrapper<MonitorMonitorTarget>().checkSqlInjection();
        if (ObjectUtil.isAllNotEmpty(monitorMonitorTargetPageParam.getSortField(), monitorMonitorTargetPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(monitorMonitorTargetPageParam.getSortOrder());
            queryWrapper.orderBy(true, monitorMonitorTargetPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(monitorMonitorTargetPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(MonitorMonitorTarget::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(MonitorMonitorTargetAddParam monitorMonitorTargetAddParam) {
        MonitorMonitorTarget monitorMonitorTarget = BeanUtil.toBean(monitorMonitorTargetAddParam, MonitorMonitorTarget.class);
        this.save(monitorMonitorTarget);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(MonitorMonitorTargetEditParam monitorMonitorTargetEditParam) {
        MonitorMonitorTarget monitorMonitorTarget = this.queryEntity(monitorMonitorTargetEditParam.getId());
        BeanUtil.copyProperties(monitorMonitorTargetEditParam, monitorMonitorTarget);
        this.updateById(monitorMonitorTarget);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<MonitorMonitorTargetIdParam> monitorMonitorTargetIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(monitorMonitorTargetIdParamList, MonitorMonitorTargetIdParam::getId));
    }

    @Override
    public MonitorMonitorTarget detail(MonitorMonitorTargetIdParam monitorMonitorTargetIdParam) {
        return this.queryEntity(monitorMonitorTargetIdParam.getId());
    }

    @Override
    public MonitorMonitorTarget queryEntity(String id) {
        MonitorMonitorTarget monitorMonitorTarget = this.getById(id);
        if (ObjectUtil.isEmpty(monitorMonitorTarget)) {
            throw new CommonException("采集器与监控对象关系不存在，id值为：{}", id);
        }
        return monitorMonitorTarget;
    }
}
