/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.monitortarget.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.coldchain.modular.monitortarget.entity.MonitorTarget;
import vip.xiaonuo.coldchain.modular.monitortarget.param.*;
import vip.xiaonuo.coldchain.modular.monitortarget.service.MonitorTargetService;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;

import java.util.List;

/**
 * 监控对象管理控制器
 *
 * @author 黄渊昊
 * @date 2024/11/13 16:56
 */
@Tag(name = "监控对象管理控制器")
@RestController
@Validated
public class MonitorTargetController {

    @Resource
    private MonitorTargetService monitorTargetService;

    /**
     * 获取监控对象管理分页
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:56
     */
    @Operation(summary = "获取监控对象管理分页")
    @SaCheckPermission("/coldchain/monitortarget/page")
    @GetMapping("/coldchain/monitortarget/page")
    public CommonResult<Page<MonitorTarget>> page(MonitorTargetPageParam monitorTargetPageParam) {
        return CommonResult.data(monitorTargetService.page(monitorTargetPageParam));
    }

/*    @Operation(summary = "根据房间获取监控对象管理分页")
    @SaCheckPermission("/coldchain/monitortarget/pageWithRoom")
    @GetMapping("/coldchain/monitortarget/pageWithRoom")
    public CommonResult<Page<MonitorTarget>> pageWithRoom(MonitorTargetPageParam monitorTargetPageParam) {
        return CommonResult.data(monitorTargetService.page(monitorTargetPageParam));
    }*/

    /**
     * 添加监控对象管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:56
     */
    @Operation(summary = "添加监控对象管理")
    @CommonLog("添加监控对象管理")
    @SaCheckPermission("/coldchain/monitortarget/add")
    @PostMapping("/coldchain/monitortarget/add")
    public CommonResult<String> add(@RequestBody @Valid MonitorTargetAddParam monitorTargetAddParam) {
        monitorTargetService.add(monitorTargetAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑监控对象管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:56
     */
    @Operation(summary = "编辑监控对象管理")
    @CommonLog("编辑监控对象管理")
    @SaCheckPermission("/coldchain/monitortarget/edit")
    @PostMapping("/coldchain/monitortarget/edit")
    public CommonResult<String> edit(@RequestBody @Valid MonitorTargetEditParam monitorTargetEditParam) {
        monitorTargetService.edit(monitorTargetEditParam);
        return CommonResult.ok();
    }

    /**
     * 批量配置报警接收人
     */



    /**
     * 根据房间添加监控对象管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:56
     */
    @Operation(summary = "根据房间添加监控对象管理")
    @CommonLog("根据房间添加监控对象管理")
    @SaCheckPermission("/coldchain/monitortarget/addWithRoom")
    @PostMapping("/coldchain/monitortarget/addWithRoom")
    public CommonResult<String> addWithRoom(@RequestBody @Valid MonitorTargetAddWithRoomParam monitorTargetAddWithRoomParam) {
        monitorTargetService.addWithRoom(monitorTargetAddWithRoomParam);
        return CommonResult.ok();
    }

    /**
     * 根据房间编辑监控对象管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:56
     */
    @Operation(summary = "根据房间编辑监控对象管理")
    @CommonLog("根据房间编辑监控对象管理")
    @SaCheckPermission("/coldchain/monitortarget/editWithRoom")
    @PostMapping("/coldchain/monitortarget/editWithRoom")
    public CommonResult<String> editWithRoom(@RequestBody @Valid MonitorTargetEditWithRoomParam monitorTargetEditWithRoomParam) {
        monitorTargetService.editWithRoom(monitorTargetEditWithRoomParam);
        return CommonResult.ok();
    }

    /**
     * 删除监控对象管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:56
     */
    @Operation(summary = "删除监控对象管理")
    @CommonLog("删除监控对象管理")
    @SaCheckPermission("/coldchain/monitortarget/delete")
    @PostMapping("/coldchain/monitortarget/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                       List<MonitorTargetIdParam> monitorTargetIdParamList) {
        monitorTargetService.delete(monitorTargetIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取监控对象管理详情
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:56
     */
    @Operation(summary = "获取监控对象管理详情")
    @SaCheckPermission("/coldchain/monitortarget/detail")
    @GetMapping("/coldchain/monitortarget/detail")
    public CommonResult<MonitorTarget> detail(@Valid MonitorTargetIdParam monitorTargetIdParam) {
        return CommonResult.data(monitorTargetService.detail(monitorTargetIdParam));
    }

    /**
     * 根据用户获取监控对象列表
     */
    @Operation(summary = "根据用户获取监控对象列表")
    @SaCheckPermission("/coldchain/monitortarget/getListByUser")
    @GetMapping("/coldchain/monitortarget/getListByUser")
    public CommonResult<List<MonitorTarget>> getListByUser(String userId) {
        return CommonResult.data(monitorTargetService.getListByUser(userId));
    }

    /**
     * 根据用户获取监控对象管理分页
     */
    @Operation(summary = "获取监控对象管理分页")
    @SaCheckPermission("/coldchain/monitortarget/getPageByUser")
    @GetMapping("/coldchain/monitortarget/getPageByUser")
    public CommonResult<Page<MonitorTarget>> getPageByUser(MonitorTargetPageParam monitorTargetPageParam) {
        return CommonResult.data(monitorTargetService.getPageByUser(monitorTargetPageParam));
    }

    /**
     * 获取监控对象列表
     */
    @Operation(summary = "获取监控对象列表")
    @SaCheckPermission("/coldchain/monitortarget/list")
    @GetMapping("/coldchain/monitortarget/list")
    public CommonResult<List<MonitorTarget>> list(Boolean isAll,String sensorType) {
        return CommonResult.data(monitorTargetService.myList(isAll));
    }

    /**
     * 下载Excel模板
     */
    @Operation(summary = "下载Excel模板")
    @GetMapping("/coldchain/monitortarget/downloadTemplate")
    public CommonResult<String> downloadTemplate(HttpServletResponse response) {
        monitorTargetService.downloadTemplate(response);
        return CommonResult.ok();
    }

    /**
     * 导入数据
     */
    @Operation(summary = "导入数据")
    @PostMapping("/coldchain/monitortarget/import")
    public CommonResult<String> importMonitorDevice(MultipartFile file) {
        monitorTargetService.importMonitorDevice(file);
        return CommonResult.ok();
    }
}
