/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.monitortarget.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.google.common.collect.Lists;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Getter;
import lombok.Setter;
import vip.xiaonuo.coldchain.core.alarm.bean.NotificationChannel;
import vip.xiaonuo.coldchain.core.alarm.bean.SensorAlarmUser;
import vip.xiaonuo.coldchain.core.handler.NotificationChannelListTypeHandler;
import vip.xiaonuo.coldchain.core.handler.SensorAlarmUserTypeHandler;
import vip.xiaonuo.coldchain.modular.monitortargetregion.entity.MonitorTargetRegion;
import vip.xiaonuo.common.pojo.OrgEntity;

import java.util.Date;
import java.util.List;

/**
 * 监控对象管理实体
 *
 * @author 黄渊昊
 * @date 2024/11/13 16:56
 **/
@Getter
@Setter
@TableName(value = "monitor_target", autoResultMap = true)
public class MonitorTarget extends OrgEntity {

    /**
     * ID
     */
    @TableId
    @Schema(description = "ID")
    private String id;

    /**
     * 名称
     */
    @Schema(description = "名称")
    private String name;

    /**
     * 状态（1：正常，2：闲置）
     */
    @Schema(description = "状态（1：正常，2：闲置）")
    private String status = "2";

    /**
     * 监控位置区域
     */
    @Schema(description = "监控位置区域")
    private String monitorPoint;

    /**
     * 设备区域列表
     */
    @TableField(exist = false)
    private List<MonitorTargetRegion> monitorTargetRegionList;

    /**
     * 最后心跳时间
     */
    @Schema(description = "最后心跳时间")
    private Date lastHeartbeatTime;

    /**
     * 最近登录时间
     */
    @Schema(description = "最近登录时间")
    private Date lastLoginTime;

    /**
     * 房间id
     */
    @Schema(description = "房间id")
    private String roomId;

    /**
     * 房间名
     */
    @Schema(description = "房间名")
    @TableField(exist = false)
    private String roomName;

    /**
     * 告警接收人，存储告警通知的接收用户信息
     */
    @TableField(value = "alarm_users", typeHandler = SensorAlarmUserTypeHandler.class)
    @Schema(description = "告警接收人，存储告警通知的接收用户信息")
    private List<SensorAlarmUser> alarmUsers = Lists.newArrayList();

    /**
     * 用户的通知渠道设置，支持选择接收告警的多个渠道，如短信、邮件、APP通知等
     */
    @Schema(description = "用户的通知渠道设置，支持选择接收告警的多个渠道，如短信、邮件、APP通知等")
    @TableField(value = "notification_channel", typeHandler = NotificationChannelListTypeHandler.class)
    private List<NotificationChannel> notificationChannel = List.of(NotificationChannel.WECHAT);
}
