/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.monitortarget.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.coldchain.core.alarm.bean.SensorAlarmUser;
import vip.xiaonuo.coldchain.modular.monitortarget.entity.MonitorTarget;
import vip.xiaonuo.coldchain.modular.monitortarget.enums.MonitorStatusEnum;
import vip.xiaonuo.coldchain.modular.monitortarget.param.*;
import vip.xiaonuo.coldchain.modular.targetroom.entity.TargetRoom;

import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * 监控对象管理Service接口
 *
 * @author 黄渊昊
 * @date 2024/11/13 16:56
 **/
public interface MonitorTargetService extends IService<MonitorTarget> {

    /**
     * 获取监控对象管理分页
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:56
     */
    Page<MonitorTarget> page(MonitorTargetPageParam monitorTargetPageParam);

    /**
     * 根据房间获取监控对象管理分页
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:56
     */
    Page<MonitorTarget> pageWithRoom(MonitorTargetPageParam monitorTargetPageParam);

    /**
     * 添加监控对象管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:56
     */
    void add(MonitorTargetAddParam monitorTargetAddParam);

    /**
     * 编辑监控对象管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:56
     */
    void edit(MonitorTargetEditParam monitorTargetEditParam);

    /**
     * 批量设置预警接收人
     */
    void batchEditAlarmUsers(MonitorTargetBatchEditWithRoomParam batchEditWithRoomParam);

    /**
     * 删除监控对象管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:56
     */
    void delete(List<MonitorTargetIdParam> monitorTargetIdParamList);

    /**
     * 获取监控对象管理详情
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:56
     */
    MonitorTarget detail(MonitorTargetIdParam monitorTargetIdParam);

    /**
     * 获取监控对象管理详情
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:56
     **/
    MonitorTarget queryEntity(String id);

    /**
     * 根据用户获取监控对象列表
     */
    List<MonitorTarget> getListByUser(String userId);

    /**
     * 根据用户获取监控对象管理分页
     */
    Page<MonitorTarget> getPageByUser(MonitorTargetPageParam monitorTargetPageParam);

    /**
     * 根据设备对象状态统计数据
     */
    TargetStatus getTargetCount(String name,String roomId);

    /**
     * 更新最后一次心跳时间
     */
    boolean updateLastHeartbeatTimeByDeviceId(Integer deviceId, Date timestamp);

    /**
     * 更新最后一次登录时间
     */
    boolean updateLastLoginTimeByDeviceId(Integer deviceId, Date timestamp);

    /**
     * 更新设备状态
     * @param deviceCode
     * @return
     */

    boolean updateStatusByDeviceCode(String deviceCode, Integer route, MonitorStatusEnum monitorStatusEnum);

    /**
     * 获取监控对象列表
     * @param isAll
     * @return
     */
    List<MonitorTarget> myList(Boolean isAll);

    /**
     * 带房间的新增监控对象
     * @param monitorTargetAddWithRoomParam
     */
    void addWithRoom(MonitorTargetAddWithRoomParam monitorTargetAddWithRoomParam);

    /**
     * 带房间的编辑监控对象
     */
    void editWithRoom(MonitorTargetEditWithRoomParam monitorTargetEditWithRoomParam);

    List<MonitorTarget> getTargetListByRoom(String roomId);

    TargetRoom getRoomByMonitorTargetId(String monitorTargetId);

    void updateStatusbatch(Set<String> allSensorCodes);

    Page<MonitorTarget> selectMonitorTargetByPage(MonitorTargetPageParam monitorTargetPageParam);

    /**
     * 下载导入模版
     * @param response
     */
    void downloadTemplate(HttpServletResponse response);

    /**
     * 导入数据
     * @param file
     * @return
     */
    void importMonitorDevice(MultipartFile file);
}
