/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.monitortargetregion.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.lang.tree.Tree;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.coldchain.modular.monitortargetregion.entity.MonitorTargetRegion;
import vip.xiaonuo.coldchain.modular.monitortargetregion.param.MonitorTargetRegionAddParam;
import vip.xiaonuo.coldchain.modular.monitortargetregion.param.MonitorTargetRegionEditParam;
import vip.xiaonuo.coldchain.modular.monitortargetregion.param.MonitorTargetRegionIdParam;
import vip.xiaonuo.coldchain.modular.monitortargetregion.param.MonitorTargetRegionPageParam;
import vip.xiaonuo.coldchain.modular.monitortargetregion.service.MonitorTargetRegionService;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;

import java.util.List;

/**
 * 区域管理控制器
 *
 * @author 黄渊昊
 * @date 2024/11/13 16:57
 */
@Tag(name = "区域管理控制器")
@RestController
@Validated
public class MonitorTargetRegionController {

    @Resource
    private MonitorTargetRegionService monitorTargetRegionService;

    /**
     * 获取区域管理分页
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:57
     */
    @Operation(summary = "获取区域管理分页")
    @SaCheckPermission("/coldchain/monitortargetregion/page")
    @GetMapping("/coldchain/monitortargetregion/page")
    public CommonResult<Page<MonitorTargetRegion>> page(MonitorTargetRegionPageParam monitorTargetRegionPageParam) {
        CommonResult<Page<MonitorTargetRegion>> data = CommonResult.data(monitorTargetRegionService.page(monitorTargetRegionPageParam));
        return data;
    }

    /**
     * 添加区域管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:57
     */
    @Operation(summary = "添加区域管理")
    @CommonLog("添加区域管理")
    @SaCheckPermission("/coldchain/monitortargetregion/add")
    @PostMapping("/coldchain/monitortargetregion/add")
    public CommonResult<String> add(@RequestBody @Valid MonitorTargetRegionAddParam monitorTargetRegionAddParam) {
        monitorTargetRegionService.add(monitorTargetRegionAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑区域管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:57
     */
    @Operation(summary = "编辑区域管理")
    @CommonLog("编辑区域管理")
    @SaCheckPermission("/coldchain/monitortargetregion/edit")
    @PostMapping("/coldchain/monitortargetregion/edit")
    public CommonResult<String> edit(@RequestBody @Valid MonitorTargetRegionEditParam monitorTargetRegionEditParam) {
        monitorTargetRegionService.edit(monitorTargetRegionEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除区域管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:57
     */
    @Operation(summary = "删除区域管理")
    @CommonLog("删除区域管理")
    @SaCheckPermission("/coldchain/monitortargetregion/delete")
    @PostMapping("/coldchain/monitortargetregion/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                       List<MonitorTargetRegionIdParam> monitorTargetRegionIdParamList) {
        monitorTargetRegionService.delete(monitorTargetRegionIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取区域管理详情
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:57
     */
    @Operation(summary = "获取区域管理详情")
    @SaCheckPermission("/coldchain/monitortargetregion/detail")
    @GetMapping("/coldchain/monitortargetregion/detail")
    public CommonResult<MonitorTargetRegion> detail(@Valid MonitorTargetRegionIdParam monitorTargetRegionIdParam) {
        return CommonResult.data(monitorTargetRegionService.detail(monitorTargetRegionIdParam));
    }

    /**
     * 获取区域树
     */
    @Operation(summary = "获取区域树")
    @SaCheckPermission("/coldchain/monitortargetregion/tree")
    @GetMapping("/coldchain/monitortargetregion/tree")
    public CommonResult<List<Tree<String>>> tree() {
        return CommonResult.data(monitorTargetRegionService.tree());
    }

    /**
     * 根据监控对象id获取监控点位列表
     */
    @Operation(summary = "根据监控对象id获取监控点位列表")
    @SaCheckPermission("/coldchain/monitortargetregion/listByTargetId")
    @GetMapping("/coldchain/monitortargetregion/listByTargetId")
    public CommonResult<List<MonitorTargetRegion>> listByTargetId(String targetId) {
        return CommonResult.data(monitorTargetRegionService.listByTargetId(targetId));
    }

    /**
     * 根据监测设备id获取监测点位列表
     */
    @Operation(summary = "根据监测设备id获取监测点位列表")
    @SaCheckPermission("/coldchain/monitordevice/getRegionByDeviceId")
    @GetMapping("/coldchain/monitordevice/getRegionByDeviceId")
    public CommonResult<List<MonitorTargetRegion>> listByDeviceId(String deviceId) {
        return CommonResult.data(monitorTargetRegionService.listByDeviceId(deviceId));
    }

    /**
     * 根据监控对象id获取监测设备编号列表
     */
    @Operation(summary = "根据监控对象id获取监测设备编号列表")
    @SaCheckPermission("/coldchain/monitordevice/getDeviceCodeByTargetId")
    @GetMapping("/coldchain/monitordevice/getDeviceCodeByTargetId")
    public CommonResult<List<String>> listByDeviceCodeByTargetId(String targetId) {
        return CommonResult.data(monitorTargetRegionService.listByDeviceCodeByTargetId(targetId));
    }
}
