/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.monitortargetregion.service;

import cn.hutool.core.lang.tree.Tree;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import vip.xiaonuo.coldchain.modular.monitortargetregion.entity.MonitorTargetRegion;
import vip.xiaonuo.coldchain.modular.monitortargetregion.param.MonitorTargetRegionAddParam;
import vip.xiaonuo.coldchain.modular.monitortargetregion.param.MonitorTargetRegionEditParam;
import vip.xiaonuo.coldchain.modular.monitortargetregion.param.MonitorTargetRegionIdParam;
import vip.xiaonuo.coldchain.modular.monitortargetregion.param.MonitorTargetRegionPageParam;

import java.util.List;

/**
 * 区域管理Service接口
 *
 * @author 黄渊昊
 * @date 2024/11/13 16:57
 **/
public interface MonitorTargetRegionService extends IService<MonitorTargetRegion> {

    /**
     * 获取区域管理分页
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:57
     */
    Page<MonitorTargetRegion> page(MonitorTargetRegionPageParam monitorTargetRegionPageParam);

    /**
     * 添加区域管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:57
     */
    void add(MonitorTargetRegionAddParam monitorTargetRegionAddParam);

    /**
     * 编辑区域管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:57
     */
    void edit(MonitorTargetRegionEditParam monitorTargetRegionEditParam);

    /**
     * 删除区域管理
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:57
     */
    void delete(List<MonitorTargetRegionIdParam> monitorTargetRegionIdParamList);

    /**
     * 获取区域管理详情
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:57
     */
    MonitorTargetRegion detail(MonitorTargetRegionIdParam monitorTargetRegionIdParam);

    /**
     * 获取区域管理详情
     *
     * @author 黄渊昊
     * @date 2024/11/13 16:57
     **/
    MonitorTargetRegion queryEntity(String id);

    /**
     * 根据targetId获取region列表
     */
    List<MonitorTargetRegion> getRegionListByTargetId(String targetId);

    /**
     * 获取区域树
     */
    List<Tree<String>> tree();

    /**
     * 根据deviceId获取targetId列表
     */
    List<String> getTargetIdListByDeviceId(String deviceCode);

    /**
     * 根据监控对象id获取监控点位列表
     */
    List<MonitorTargetRegion> listByTargetId(String targetId);


    MonitorTargetRegion findOneByDeviceCodeAndSensorNo(String deviceCode,Integer sensorNo);

    /**
     * 获取已使用总路数
     */
    Integer getUsedSensorCount();

    /**
     * 根据检测设备id获取监测点位列表
     */
    List<MonitorTargetRegion> listByDeviceId(String deviceId);

    /**
     * 根据监控对象id获取监测设备编号列表
     */
    List<String> listByDeviceCodeByTargetId(String targetId);
}
