/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.targetroom.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.coldchain.modular.targetroom.entity.TargetRoom;
import vip.xiaonuo.coldchain.modular.targetroom.param.TargetRoomAddParam;
import vip.xiaonuo.coldchain.modular.targetroom.param.TargetRoomEditParam;
import vip.xiaonuo.coldchain.modular.targetroom.param.TargetRoomIdParam;
import vip.xiaonuo.coldchain.modular.targetroom.param.TargetRoomPageParam;
import vip.xiaonuo.coldchain.modular.targetroom.service.TargetRoomService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 房间配置控制器
 *
 * @author hyh
 * @date  2024/12/31 00:05
 */
@Tag(name = "房间配置控制器")
@RestController
@Validated
public class TargetRoomController {

    @Resource
    private TargetRoomService targetRoomService;

    /**
     * 获取房间配置分页
     *
     * @author hyh
     * @date  2024/12/31 00:05
     */
    @Operation(summary = "获取房间配置分页")
    @SaCheckPermission("/coldchain/targetroom/page")
    @GetMapping("/coldchain/targetroom/page")
    public CommonResult<Page<TargetRoom>> page(TargetRoomPageParam targetRoomPageParam) {
        return CommonResult.data(targetRoomService.page(targetRoomPageParam));
    }

    /**
     * 添加房间配置
     *
     * @author hyh
     * @date  2024/12/31 00:05
     */
    @Operation(summary = "添加房间配置")
    @CommonLog("添加房间配置")
    @SaCheckPermission("/coldchain/targetroom/add")
    @PostMapping("/coldchain/targetroom/add")
    public CommonResult<String> add(@RequestBody @Valid TargetRoomAddParam targetRoomAddParam) {
        targetRoomService.add(targetRoomAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑房间配置
     *
     * @author hyh
     * @date  2024/12/31 00:05
     */
    @Operation(summary = "编辑房间配置")
    @CommonLog("编辑房间配置")
    @SaCheckPermission("/coldchain/targetroom/edit")
    @PostMapping("/coldchain/targetroom/edit")
    public CommonResult<String> edit(@RequestBody @Valid TargetRoomEditParam targetRoomEditParam) {
        targetRoomService.edit(targetRoomEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除房间配置
     *
     * @author hyh
     * @date  2024/12/31 00:05
     */
    @Operation(summary = "删除房间配置")
    @CommonLog("删除房间配置")
    @SaCheckPermission("/coldchain/targetroom/delete")
    @PostMapping("/coldchain/targetroom/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<TargetRoomIdParam> targetRoomIdParamList) {
        targetRoomService.delete(targetRoomIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取房间配置详情
     *
     * @author hyh
     * @date  2024/12/31 00:05
     */
    @Operation(summary = "获取房间配置详情")
    @SaCheckPermission("/coldchain/targetroom/detail")
    @GetMapping("/coldchain/targetroom/detail")
    public CommonResult<TargetRoom> detail(@Valid TargetRoomIdParam targetRoomIdParam) {
        return CommonResult.data(targetRoomService.detail(targetRoomIdParam));
    }

    /**
     * 获取房间列表
     */
    @Operation(summary = "获取房间列表")
    @SaCheckPermission("/coldchain/targetroom/list")
    @GetMapping("/coldchain/targetroom/list")
    public CommonResult<List<TargetRoom>> list() {
        return CommonResult.data(targetRoomService.myList());
    }
}
